@@ -1,13 +1,14 @@
 package App::Netdisco::Daemon::Worker::Poller::Device;
 
 use Dancer qw/:moose :syntax :script/;
-use Dancer::Plugin::DBIC 'schema';
 
 use App::Netdisco::Util::SNMP 'snmp_connect';
 use App::Netdisco::Util::Device qw/get_device is_discoverable/;
 use App::Netdisco::Core::Discover ':all';
 use App::Netdisco::Daemon::Util ':all';
+use App::Netdisco::JobQueue qw/jq_queued jq_insert/;
 
+use Dancer::Plugin::DBIC 'schema';
 use NetAddr::IP::Lite ':lower';
 
 use Role::Tiny;
@@ -17,30 +18,19 @@ use namespace::clean;
 sub discoverall {
   my ($self, $job) = @_;
 
-  my $devices = schema('netdisco')->resultset('Device')->get_column('ip');
-  my $jobqueue = schema('netdisco')->resultset('Admin');
+  my %queued = map {$_ => 1} jq_queued('discover');
+  my @devices = schema('netdisco')->resultset('Device')
+    ->get_column('ip')->all;
+  my @filtered_devices = grep {!exists $queued{$_}} @devices;
 
-  schema('netdisco')->txn_do(sub {
-    # clean up user submitted jobs older than 1min,
-    # assuming skew between schedulers' clocks is not greater than 1min
-    $jobqueue->search({
-        action => 'discover',
-        status => 'queued',
-        entered => { '<' => \"(now() - interval '1 minute')" },
-    })->delete;
-
-    # is scuppered by any user job submitted in last 1min (bad), or
-    # any similar job from another scheduler (good)
-    $jobqueue->populate([
+  jq_insert([
       map {{
           device => $_,
           action => 'discover',
-          status => 'queued',
           username => $job->username,
           userip => $job->userip,
-      }} ($devices->all)
-    ]);
-  });
+      }} (@filtered_devices)
+  ]);
 
   return job_done("Queued discover job for all devices");
 }
@@ -51,7 +41,6 @@ sub discover {
 
   my $host = NetAddr::IP::Lite->new($job->device);
   my $device = get_device($host->addr);
-  my $jobqueue = schema('netdisco')->resultset('Admin');
 
   if ($device->ip eq '0.0.0.0') {
       return job_error("discover failed: no device param (need -d ?)");
@@ -71,6 +60,7 @@ sub discover {
       return job_error("discover failed: could not SNMP connect to $host");
   }
 
+  $device = set_canonical_ip($device, $snmp);
   store_device($device, $snmp);
   store_interfaces($device, $snmp);
   store_wireless($device, $snmp);
@@ -82,26 +72,20 @@ sub discover {
   # if requested, and the device has not yet been arpniped/macsucked, queue now
   if ($device->in_storage and $job->subaction and $job->subaction eq 'with-nodes') {
       if (!defined $device->last_macsuck) {
-          schema('netdisco')->txn_do(sub {
-            $jobqueue->create({
+          jq_insert({
               device => $device->ip,
               action => 'macsuck',
-              status => 'queued',
               username => $job->username,
               userip => $job->userip,
-            });
           });
       }
 
       if (!defined $device->last_arpnip) {
-          schema('netdisco')->txn_do(sub {
-            $jobqueue->create({
+          jq_insert({
               device => $device->ip,
               action => 'arpnip',
-              status => 'queued',
               username => $job->username,
               userip => $job->userip,
-            });
           });
       }
   }
